@@ -60,6 +60,13 @@
  * report bugs, too!), but don't _expect_ them to work.
  */
 
+/*
+ * Forcefully disable double-buffering for xft. On some card/driver combination
+ * enabling it slightly decreases performance, on most it greatly helps it.
+ * The slowdown is small, so it should normally be enabled.
+ */
+/* #define FORCE_UNBUFFERED_XFT */
+
 /*
  * The cursor blink interval, in seconds.
  */
@@ -136,7 +143,7 @@
  * Define to remove support for XCopyArea () support.  XCopyArea () is useful
  * for scrolling on non-local X displays
  */
-#undef NO_SLOW_LINK_SUPPORT
+#define NO_SLOW_LINK_SUPPORT
 
 /*
  * Printer pipe which will be used for emulation of attached vt100 printer
@@ -384,10 +391,15 @@
 
 /*
  * Enable the linux yield/usleep hack, which can dramatically improve
- * performance without hurting. See command.C for details.
+ * performance by working around the linux kernel tty ratelimit bug.
+ * Unfortunately, it seems screen is negatively affected by this on some
+ * machines, so it is disabled by default. Use freebsd or any other kernel
+ * that doesn't suffer form this bug and it will be fast either way.
+ *
+ * See command.C for details.
  */
 #if __linux__
-# define LINUX_YIELD_HACK 1
+# define LINUX_YIELD_HACK 0
 #endif
 
 #endif
