@@ -10,15 +10,16 @@
 #endif
 
 #if ENABLE_PERL
-# define ENABLE_FRILLS 1
+# define ENABLE_FRILLS    1
 # define ENABLE_COMBINING 1
 #endif
 
 #if ENABLE_FRILLS
-# define ENABLE_XEMBED 1
-# define ENABLE_EWMH   1
+# define ENABLE_XEMBED        1
+# define ENABLE_EWMH          1
 # define ENABLE_XIM_ONTHESPOT 1
-# define CURSOR_BLINK  1
+# define CURSOR_BLINK         1
+# define OPTION_HC            1
 #else
 # define ENABLE_MINIMAL 1
 #endif
@@ -71,19 +72,8 @@
 typedef char *XPointer;
 #endif
 
-#ifdef HAVE_TERMIOS_H
-# include <termios.h>
+#include <termios.h>
 typedef struct termios ttymode_t;
-#else
-# include <sgtty.h>
-typedef struct {
-  struct sgttyb   sg;
-  struct tchars   tc;
-  struct ltchars  lc;
-  int             line;
-  int             local;
-} ttymode_t;
-#endif
 
 #ifdef XPM_BACKGROUND
 # ifdef XPM_INC_X11
@@ -521,6 +511,8 @@ enum {
  */
 
 enum colour_list {
+  Color_none = -2,
+  Color_transparent = -1,
   Color_fg = 0,
   Color_bg,
   minCOLOR,                   /* 2 */
@@ -683,10 +675,9 @@ enum {
  */
 #define dLocal(type,name)       type const name = this->name
 
-// for speed reasons, we assume that all latin1 characters
-// are single-width (the first unicode combining character
-// is actually 0x300, but ascii is what matters most).
-#define WCWIDTH(c) ((c) & ~0xff ? wcwidth (c) : 1)
+// for speed reasons, we assume that all codepoints 32 to 126 are
+// single-width.
+#define WCWIDTH(c)		(IN_RANGE_INC (c, 0x20, 0x7e) ? 1 : wcwidth (c))
 
 /* convert pixel dimensions to row/column values.  Everything as int32_t */
 #define Pixel2Col(x)            Pixel2Width((int32_t)(x))
@@ -699,9 +690,13 @@ enum {
 #define Height2Pixel(n)         ((int32_t)(n) * (int32_t)fheight)
 
 #define OPTION(opt)		(options & (opt))
-#define DEFAULT_OPTIONS		(Opt_scrollBar | Opt_scrollTtyOutput \
-				 | Opt_jumpScroll | Opt_secondaryScreen \
-				 | Opt_pastableTabs | Opt_intensityStyles)
+#define DEFAULT_OPTIONS		(Opt_scrollBar		\
+                                | Opt_scrollTtyOutput	\
+				| Opt_jumpScroll	\
+                                | Opt_secondaryScreen	\
+                                | Opt_secondaryScroll	\
+				| Opt_pastableTabs	\
+                                | Opt_intensityStyles)
 
 // for m >= -n, ensure remainder lies between 0..n-1
 #define MOD(m,n) (((m) + (n)) % (n))
@@ -775,8 +770,8 @@ enum {
 # define XPMClearArea(a, b, c, d, e, f, g)
 #endif
 
-typedef callback1<void, const char *> log_callback;
-typedef callback1<int, int> getfd_callback;
+typedef callback<void (const char *)> log_callback;
+typedef callback<int (int)> getfd_callback;
 
 #define SET_LOCALE(locale) rxvt_set_locale (locale)
 extern bool rxvt_set_locale (const char *locale) NOTHROW;
@@ -788,6 +783,8 @@ extern void rxvt_pop_locale () NOTHROW;
 #define LINE_LONGER     0x0001 // line is continued on the next row
 #define LINE_FILTERED   0x0002 // line has been filtered
 #define LINE_COMPRESSED 0x0004 // line has been compressed (NYI)
+#define LINE_FILTER     0x0008 // line needs to be filtered before display (NYI)
+#define LINE_BIDI       0x0010 // line needs bidi (NYI)
 
 struct line_t {
    text_t *t; // terminal the text
@@ -910,8 +907,6 @@ struct rxvt_term : zero_initialized, rxvt_vars, rxvt_screen {
   unsigned char   want_refresh:1,
 #ifdef TRANSPARENT
                   want_full_refresh:1,	/* awaiting full screen refresh      */
-#endif
-#if defined(XPM_BACKGROUND) || defined(TRANSPARENT)
                   am_transparent:1,	/* is a transparent term             */
                   am_pixmap_trans:1, 	/* transparency w/known root pixmap  */
 #endif
@@ -1100,7 +1095,7 @@ struct rxvt_term : zero_initialized, rxvt_vars, rxvt_screen {
     XSelectInput (dpy, vt, vt_emask | vt_emask_perl | vt_emask_xim);
   }
 
-#if TRANSPARENT
+#if TRANSPARENT || ENABLE_PERL
   void rootwin_cb (XEvent &xev);
   xevent_watcher rootwin_ev;
 #endif
@@ -1172,8 +1167,8 @@ struct rxvt_term : zero_initialized, rxvt_vars, rxvt_screen {
   void make_current () const // make this the "currently active" urxvt instance
   {
     SET_R (this);
-    rxvt_set_locale (locale);
     set_environ (envv);
+    rxvt_set_locale (locale);
   }
 
   void init_secondary ();
@@ -1208,7 +1203,7 @@ struct rxvt_term : zero_initialized, rxvt_vars, rxvt_screen {
   bool IMisRunning ();
   void IMSendSpot ();
   bool IM_get_IC (const char *modifiers);
-  void IMSetStatusPosition ();
+  void IMSetPosition ();
 #endif
 
   void resize_scrollbar ();
@@ -1229,6 +1224,7 @@ struct rxvt_term : zero_initialized, rxvt_vars, rxvt_screen {
   void button_release (XButtonEvent &ev);
   void focus_in ();
   void focus_out ();
+  void update_fade_color (unsigned int idx);
   int check_our_parents ();
 #ifdef PRINTPIPE
   FILE *popen_printer ();
